import pygame
from pygame.locals import *

from Entity import *
from Vector2 import *
from GasCloud import *
from Trampoline import *

class Drop(Entity):
	def __init__(self, player, state, pos, color):
		Entity.__init__(self)
		
		self.state = state
		self.player = player
		self.pos = pos
		self.v = Vec2d(0, 0)
		self.color = color
		if self.color == 1:
			self.img = pygame.image.load("Assets/redDrop.png")
		elif self.color == 2:
			self.img = pygame.image.load("Assets/greenDrop.png")
		else:
			print("Color out of range")
			
		self.size = Vec2d(self.img.get_width(), self.img.get_height())
			
		self.allRect = pygame.Rect(self.pos.x, self.pos.y, self.size.x, self.size.y)
			
	def update(self, dt):
		self.v.y += 30
		self.move(dt)
		self.allRect = pygame.Rect(self.pos.x, self.pos.y, self.size.x, self.size.y)
		self.checkBounds()
		
	def move(self, dt):
		self.pos += self.v*dt
	
	def onCollision(self, entity):
		if entity == self.player:
			return
		if self.color == 1:
			tramp = Trampoline(self.state, self.pos)
			self.state.entityManager.addEntity(tramp)
			return
		if self.color == 2:
			cloud = GasCloud(self.player, self.state, self.pos)
			self.state.entityManager.addEntity(cloud)
		
		self.doomed = True
		
	def getRect(self):
		return self.allRect
		
	def render(self):
		self.state.draw(self.img, self.pos)
		
	def checkBounds(self):
		if self.pos.x < 0-self.size.x:
			self.doomed = True
		if self.pos.y < 0-self.size.y:
			self.doomed = True
		if self.pos.x > self.state.bounds.x:
			self.doomed = True
		if self.pos.y > self.state.bounds.y - self.size.y:
			if self.color == 2:
				cloud = GasCloud(self.player, self.state, self.pos)
				self.state.entityManager.addEntity(cloud)
			if self.color == 1:
				tramp = Trampoline(self.state, self.pos)
				self.state.entityManager.addEntity(tramp)
			self.doomed = True
