import pygame
from pygame.locals import *

from GameState import *
from EntityManager import *
from Player import *
from Camera import *
from Factory import *
from Exit import *
from Level1 import *
from Level2 import *
from Level3 import *
from Level4 import *
from Level5 import *
from Level6 import *

class InGame(GameState):
    def __init__(self, game):
        GameState.__init__(self, game)
        
        self.changedState = False
        
        self.RENDERSURF = pygame.Surface((1920, 1080))
        self.startPos = Vec2d(0, 0)
        
        self.player = Player(self.game, self)
		
        self.entityManager = EntityManager()
        self.entityManager.addEntity(self.player)
        self.factory = Factory(self)
        
        self.levelCounter = 1
        self.changeLevel(self.levelCounter)
		
    def changeLevel(self, number):
        self.cleanup()
        
        self.stateChanged = True
		
        if number == 1:
            self.level = Level1(self)
        elif number == 2:
            self.level = Level2(self)
        elif number == 3:
            self.level = Level3(self)
        elif number == 4:
            self.level = Level4(self)
        elif number == 5:
            self.level = Level5(self)
        elif number == 6:
            self.level = Level6(self)
		
        self.bounds = self.level.bounds
        self.player.pos = Vec2d(self.startPos.x, self.startPos.y)
        self.player.red = False
        self.player.green = False
        self.player.onGround = False
        self.player.onCloud = False
        self.player.spilling = False
		
    def handleEvents(self):
        if self.changedState == True:
            self.changedState = False
        self.events = pygame.event.get()
		
        for event in self.events:
            if event.type == QUIT:
                self.game.exit()
            if event.type == KEYUP:
                if event.key == K_ESCAPE:
                    self.exit()
		

    def update(self, dt):
        if self.changedState == True:
            return
        self.entityManager.update(dt)
		
        self.checkCollisions(self.entityManager.getMovingCollidable(), self.entityManager.getCollidable())
		
        if self.player.pos.x > self.level.bounds.x:
            self.levelCounter +=1
            if self.levelCounter <= 6:
                self.changeLevel(self.levelCounter)
            else:
                self.game.changeState(Exit(self.game))
		
        return
		
    def draw(self, img, pos, area=None):
        surf = self.RENDERSURF
        if pos.x-self.level.camera.pos.x > surf.get_width() or pos.y-self.level.camera.pos.y > surf.get_height():
            return
        surf.blit(img, (pos.x-self.level.camera.pos.x, pos.y-self.level.camera.pos.y), area)
		
		
    def render(self):
        if self.changedState == True:
            return
        self.RENDERSURF.fill(pygame.Color(255, 255, 255))
		
        for entity in self.entityManager.getRenderable():
            if entity == self.player:
                continue
            entity.render()
			
        self.player.render()
		
        for entity in self.entityManager.getRrrenderable():
            entity.rrrender()
		
        pygame.transform.scale(self.RENDERSURF, (self.game.width, self.game.height), self.game.DISPLAYSURF)
        pygame.display.flip()
		
    def exit(self):
        self.game.exit()
        return
		
    def cleanup(self):
        self.entityManager.cleanup()
        self.entityManager.addEntity(self.player)
		
		
		
    def checkCollisions(self, movable, collidable):
        for mov in movable:
            for entity in collidable:
                if not mov == entity:
                    if mov.getRect().colliderect(entity.getRect()):
                        if mov.useMask and entity.useMask:
                            mask1 = mov.getMask()
                            mask2 = entity.getMask()
                            offset = mov.pos-entity.pos
                            offset.x = int(offset.x)
                            offset.y = int(offset.y)
                            x = (mask1.overlap_area(mask2,(offset.x+1,offset.y)) - mask1.overlap_area(mask2,(offset.x-1,offset.y)))  
                            y = (mask1.overlap_area(mask2,(offset.x,offset.y+1)) - mask1.overlap_area(mask2,(offset.x,offset.y-1)))  
                            if x == 0 and y == 0:  
                                mov.onCollision(entity)
                                entity.onCollision(mov)
                        else:
                            mov.onCollision(entity)
                            entity.onCollision(mov)
							
    def getEvents(self):
        return self.events
