import pygame
from pygame.locals import *

from Entity import *
from Vector2 import *
from Player import *

class Plasma(Entity):
    def __init__(self, cannon, direction, state):
        Entity.__init__(self)
        self.cannon = cannon
        self.state = state
        self.pos = Vec2d(0, 0)
        self.direction = direction
        self.vmax = 100
        self.img = pygame.image.load("Assets/plasma.png")
        self.size = Vec2d(self.img.get_width(), self.img.get_height())
        self.updateRect()
        self.useMask = True
		
    def update(self, dt):
        self.move(dt)
        self.checkBounds()
        self.updateRect()

    def move(self, dt):
        self.pos += self.vmax*self.direction*dt
		
    def render(self):
        self.state.draw(self.img, self.pos)
		
    def onCollision(self, victim):
        if victim == self.cannon:
            return
		
        if hasattr(victim, "die"):
            victim.die()
        if isinstance(victim, Player):
            return
        self.vanish()
		
    def checkBounds(self):
        if self.pos.x < 0-self.size.x:
            self.vanish()
        if self.pos.y < 0-self.size.y:
            self.vanish()
        if self.pos.x > self.state.bounds.x:
            self.vanish()
        if self.pos.y > self.state.bounds.y - self.size.y:
            self.vanish()
			
    def vanish(self):
        self.doomed = True
		
    def getMask(self):
        return pygame.mask.from_surface(self.img)
		
    def getRect(self):
        return self.allRect
		
    def updateRect(self):
        self.allRect = pygame.Rect(self.pos.x+0.2*self.size.x, self.pos.y+self.size.y*0.2, self.size.x*0.6, self.size.y*0.6)
