import pygame
from pygame.locals import *

from Entity import *
from Plasma import *
from Vector2 import *

class PlasmaCannon(Entity):
	def __init__(self, state):
		Entity.__init__(self)
		
		self.state = state
		self.pos = Vec2d(0, 0)
		self.img = pygame.image.load("Assets/plasmacannon.png")
		self.size = Vec2d(self.img.get_width(), self.img.get_height())
		self.direction = Vec2d(0, 1)
		
		self.fireRate = 2.0
		self.fireAmount = 1
		self.fireTime = 0
		self.fireCounter = 0
		self.plasmaSpeed = 100
		self.useMask = True
		self.mask = pygame.mask.from_surface(self.img)
		
		self.allRect = pygame.Rect(self.pos.x, self.pos.y, self.size.x, self.size.y)
	
	def initialize(self, pos, rotation, fireRate, fireAmount, plasmaSpeed):
		self.pos = pos
		
		if rotation == 0:
			self.rotation = rotation
			self.direction = Vec2d(0, -1)
		if rotation == 90:
			self.rotation = rotation
			self.direction = Vec2d(-1, 0)
			self.allRect = pygame.Rect(self.pos.x, self.pos.y, self.size.y, self.size.x)
		if rotation == 180:
			self.rotation = rotation
			self.direction = Vec2d(0, 1)
			self.allRect = pygame.Rect(self.pos.x, self.pos.y, self.size.x, self.size.y)
		if rotation == 270 or rotation == -90:
			self.rotation = rotation
			self.direction = Vec2d(1, 0)
			self.allRect = pygame.Rect(self.pos.x, self.pos.y, self.size.y, self.size.x)
			
		self.fireRate = fireRate
		self.fireAmount = fireAmount
		self.plasmaSpeed = plasmaSpeed
		
		self.img = pygame.transform.rotate(self.img, self.rotation)
		self.size = Vec2d(self.img.get_width(), self.img.get_height())
		self.mask = pygame.mask.from_surface(self.img)
		
		
	def update(self, dt):
		self.fireTime += dt
		if self.fireTime >= self.fireRate:
			if self.fireTime >= self.fireRate + self.fireCounter * 0.2:
				if self.fireCounter < self.fireAmount:
					self.fire()
					self.fireCounter += 1
				else:
					self.fireCounter = 0
					self.fireTime = 0
		
	def fire(self):
		plasma = Plasma(self, self.direction, self.state)
		
		if self.rotation == 0:
			plasma.pos = Vec2d(self.pos.x+self.size.x*0.3, self.pos.y-plasma.size.y*0.5)
		if self.rotation == 90:
			plasma.pos = Vec2d(self.pos.x-self.size.x*0.5, self.pos.y+self.size.y*0.3)
		if self.rotation == 180:
			plasma.pos = Vec2d(self.pos.x+self.size.x*0.3, self.pos.y+self.size.y*0.5)
		if self.rotation == 270:
			plasma.pos = Vec2d(self.pos.x+self.size.x*0.5, self.pos.y+self.size.y*0.3)
		
		plasma.vmax = self.plasmaSpeed
		
		self.state.entityManager.addEntity(plasma)
		
	def render(self):
		self.state.draw(self.img, self.pos)	
	
	def getRect(self):
		return self.allRect
		
	def onCollision(self, entity):
		return
		
	def getMask(self):
		return self.mask
