import pygame
from pygame.locals import *

from Vector2 import * 
from Entity import *
from Anim import *
from Drop import *
from Trampoline import *
from Drink import *

class Player(Entity):
    def __init__(self, game, state):
        Entity.__init__(self)
		
        self.game = game
        self.state = state
        
        self.killed = False
        self.facingFront = True
		
        self.onCloud = False
        self.cloud = None
		
        self.onGround = False
		
        self.pos = Vec2d((100, 800))
        self.v = Vec2d((0, 0))
        self.vx_max = 250
        self.a = Vec2d(0, 0)
        self.m = 5.0
		
        self.jumping = False
        self.jumpTime = 0
		
        self.spilling = False
        self.dropColor = 0
		
        self.red = False
        self.green = False
		
		
        self.spillAnim = Anim()
        self.spillAnim.initialize(2, 47, "Assets/spillAnim.png")
        self.spillAnim.animTweaker = 10.0
        self.walkAnim = Anim()
        self.walkAnim.initialize(2, 12, "Assets/walkAnim.png")
        self.walkAnim.looping = True
        self.walkAnim.play()
        self.size = self.walkAnim.size
        self.walkAnim.animTweaker = 5.0
        self.useMask = True
		
    def applyForce(self, x, y):
        self.a.x += x/self.m
        self.a.y += y/self.m
		
    def update(self, dt):
        if self.killed:
            self.killed = False
            self.state.changeLevel(self.state.levelCounter)
            return
		
        if not self.spilling:
            keys = pygame.key.get_pressed()
		
            for event in self.state.getEvents():
                if event.type == KEYDOWN:
                    if event.key == K_UP:
                        if self.onGround == True and self.onCloud == False:
                            self.jumping = True
                            self.jumpTime = 0
					
                if event.type == KEYUP:
                    if event.key == K_1:
                        if self.red == False:
                            return
                        if self.onGround or self.onCloud:
                            self.dropColor = 1
                            self.spillAnim.play()
                            self.spilling = True
                    if event.key == K_2:
                        if self.green == False:
                            return
                        if self.onGround or self.onCloud:
                            self.dropColor = 2
                            self.spillAnim.play()
                            self.spilling = True
                    if event.key == K_SPACE:
                        if self.onCloud:
                            self.jumpTime = 0
                            self.jump()
                            self.onCloud = False
                            self.cloud.doomed = True
                            self.cloud = None
                    if event.key == K_UP:
                        if self.jumping == True:
                            self.jump()
                            self.jumping = False
						
		
		
            if keys[K_UP]:
                if self.onCloud:
                    if self.v.y > 10:
                        self.applyForce(0, -60*self.v.y)
                    else:
                        self.applyForce(0, -1800)
				
            if keys[K_DOWN]:
                if self.onCloud:
                    if self.v.y < -10:
                        self.applyForce(0, -60*self.v.y)
                    else:
                        self.applyForce(0, 1800)
			
		
            if keys[K_LEFT] or keys[K_RIGHT]:
                if keys[K_LEFT]:
                    if self.v.x > 0:
                        self.applyForce(-60*self.v.x, 0)
                    else:
                        self.applyForce(-1800, 0)
                    self.walkAnim.update(dt, self.v.x)
                if keys[K_RIGHT]:
                    if self.v.x < 0:
                        self.applyForce(-60*self.v.x, 0)
                    else:
                        self.applyForce(1800, 0)
                    self.walkAnim.update(dt, self.v.x)
            else:
                if self.v.x > 0:
                    self.applyForce(-50*self.v.x, 0)
                if self.v.x < 0:
                    self.applyForce(-50*self.v.x, 0)
				
                self.walkAnim.pos.x = 0
			
            if not self.onCloud:
                self.applyForce(0, 4000)
            if self.onGround:
                self.v.y = 0
			
			
            if self.jumping:
                self.jumpTime += dt
                if self.jumpTime > 0.15:
                    self.jump()
                    self.jumping = False
			
            self.onGround = False
			
            self.move(dt)
            self.checkBounds()
            self.updateRects()
		
        if self.spilling:
            self.spillAnim.update(dt)
            if not self.spillAnim.running:
                self.spilling = False
                self.spillDrink()
		
    def jump(self):
        self.applyForce(0, -13000+self.jumpTime*-320000)
        self.onGround = False
		
    def move(self, dt):
        self.v += self.a * dt
		
		
        if self.v.x > self.vx_max:
            self.v.x = self.vx_max
        if self.v.x < -self.vx_max:
            self.v.x = -self.vx_max
        self.pos += self.v * dt
        self.a = Vec2d(0, 0)
		
        if self.v.x > 0 and self.v.x < 5:
            self.v.x = 0
        if self.v.x < 0 and self.v.x > -5:
            self.v.x = 0
		
        if self.v.x > 0 and not self.walkAnim.pos.y == 1:
            self.facingFront = True
            self.walkAnim.pos = Vec2d(0, 1)
            self.spillAnim.pos.y = 1
        if self.v.x < 0 and not self.walkAnim.pos.y == 0:
            self.facingFront = False
            self.walkAnim.pos = Vec2d(0, 0)
            self.spillAnim.pos.y = 0
			
        if self.onCloud:
            self.walkAnim.pos.x = 0
			
		
		
    def render(self):
        if self.spilling:
            self.spillAnim.render(self.state, self.pos)
        else:
            self.walkAnim.render(self.state, self.pos)
		
    def onCollision(self, entity):
        self.updateRects()
		
        rect = entity.getRect()
		
        if isinstance(entity, Drink):
            if entity.color == 1:
                self.red = True
            if entity.color == 2:
                self.green = True
		
        if self.topRect.colliderect(rect):
            self.pos.y = rect.bottom
            self.v.y *= -0.01
            self.updateRects()
        if self.bottomRect.colliderect(rect):
            self.pos.y = rect.top - self.size.y
			
            if isinstance(entity, Trampoline):
                self.v.y *= -2.0
            else:
                self.v.y = 0
                self.onGround = True
                self.updateRects()
        if self.leftRect.colliderect(rect):
            if isinstance(entity, Trampoline):
                self.v.y *= -2.0
            else:
                self.pos.x = rect.right
                self.v.x *= -0.01
            self.updateRects()
        if self.rightRect.colliderect(rect):
            if isinstance(entity, Trampoline):
                self.v.y *= -2.0
            else:
                self.pos.x = rect.left - self.size.x
                self.v.x *= -0.01
            self.updateRects()
		
    def checkBounds(self):
        if self.pos.x < 0:
            self.pos.x = 0
            self.v.x *= -0.01
		
        if self.pos.y > self.state.bounds.y - self.size.y:
            self.pos.y = self.state.bounds.y - self.size.y
            self.v.y = 0
            self.onGround = True
	
    def getRect(self):
        return self.allRect
		
    def updateRects(self):
        self.allRect = pygame.Rect(self.pos.x, self.pos.y, self.size.x, self.size.y)
		
        self.topRect = pygame.Rect(self.pos.x+0.2*self.size.x, self.pos.y, self.size.x*0.6, self.size.y*0.2)
        self.bottomRect = pygame.Rect(self.pos.x+0.2*self.size.x, self.pos.y+self.size.y*0.8, self.size.x*0.6, self.size.y*0.2)
        self.rightRect = pygame.Rect(self.pos.x+0.8*self.size.x, self.pos.y+0.2*self.size.y, self.size.x*0.2, self.size.y*0.6)
        self.leftRect = pygame.Rect(self.pos.x, self.pos.y+self.size.y*0.2, self.size.x*0.2, self.size.y*0.6)
			
    def die(self):
       self.killed = True		
    def getMask(self):
        surf = self.walkAnim.img.subsurface(pygame.Rect(int(self.walkAnim.pos.x)*self.size.x, int(self.walkAnim.pos.y)*self.size.y, self.size.x, self.size.y))
        return pygame.mask.from_surface(surf, 250)
		
    def spillDrink(self):
        if self.facingFront:
            pos = Vec2d(self.pos.x+1.3*self.size.x, self.pos.y+0.3*self.size.y)
        else:
            pos = Vec2d(self.pos.x-0.3*self.size.x, self.pos.y+0.3*self.size.y)
        drop = Drop(self, self.state, pos, self.dropColor)
        self.state.entityManager.addEntity(drop)
